@@ -10,7 +10,6 @@ pub(crate) use dump_inputs::dump_inputs;
 use std::fs;
 use std::io::{self, Write};
 use std::path::PathBuf;
-use std::sync::Arc;
 
 use crate::cache::Cache;
 use crate::options::Config;
@@ -20,7 +19,7 @@ use lychee_lib::{Client, Request};
 /// Parameters passed to every command
 pub(crate) struct CommandParams<S: futures::Stream<Item = Result<Request, RequestError>>> {
     pub(crate) client: Client,
-    pub(crate) cache: Arc<Cache>,
+    pub(crate) cache: Cache,
     pub(crate) requests: S,
     pub(crate) cfg: Config,
 }
