@@ -22,8 +22,8 @@ impl ResponseFormatter for PlainFormatter {
 mod plain_tests {
     use super::*;
     use http::StatusCode;
-    use lychee_lib::Redirects;
     use lychee_lib::{ErrorKind, Status, Uri};
+    use lychee_lib::{Redirect, Redirects};
     use test_utils::mock_response_body;
 
     #[test]
@@ -62,19 +62,19 @@ mod plain_tests {
     #[test]
     fn test_format_response_with_redirect_status() {
         let formatter = PlainFormatter;
+        let mut redirects = Redirects::new("https://from.dev".try_into().unwrap());
+        redirects.push(Redirect {
+            url: "https://to.dev".try_into().unwrap(),
+            code: StatusCode::PERMANENT_REDIRECT,
+        });
+
         let body = mock_response_body!(
-            Status::Redirected(
-                StatusCode::MOVED_PERMANENTLY,
-                Redirects::from(vec![
-                    "https://from.dev".try_into().unwrap(),
-                    "https://to.dev".try_into().unwrap(),
-                ]),
-            ),
+            Status::Redirected(StatusCode::MOVED_PERMANENTLY, redirects),
             "https://example.com/redirect",
         );
         assert_eq!(
             formatter.format_response(&body),
-            "[301] https://example.com/redirect | Redirect: Followed 1 redirect resolving to the final status of: Moved Permanently. Redirects: https://from.dev/ --> https://to.dev/"
+            "[301] https://example.com/redirect | Redirect: Followed 1 redirect resolving to the final status of: Moved Permanently. Redirects: https://from.dev/ --[308]--> https://to.dev/"
         );
     }
 
