@@ -6,8 +6,13 @@ use std::{
     time::Duration,
 };
 
-use crate::formatters::color::{BOLD_GREEN, BOLD_PINK, BOLD_YELLOW, DIM, NORMAL, color};
-use crate::{formatters::get_response_formatter, options, stats::ResponseStats};
+use crate::formatters::{
+    color::{BOLD_GREEN, BOLD_PINK, BOLD_YELLOW, DIM, NORMAL, color},
+    get_response_formatter,
+    host_stats::CompactHostStats,
+    stats::{OutputStats, ResponseStats},
+};
+use crate::options;
 
 use super::StatsFormatter;
 
@@ -107,87 +112,54 @@ impl Compact {
 }
 
 impl StatsFormatter for Compact {
-    fn format(&self, stats: ResponseStats) -> Result<Option<String>> {
-        let compact = CompactResponseStats {
-            stats,
+    fn format(&self, stats: OutputStats) -> Result<String> {
+        let response_stats = CompactResponseStats {
+            stats: stats.response_stats,
             mode: self.mode.clone(),
         };
-        Ok(Some(compact.to_string()))
+        let host_stats = CompactHostStats {
+            host_stats: stats.host_stats,
+        };
+
+        Ok(format!("{response_stats}\n{host_stats}"))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use crate::formatters::stats::StatsFormatter;
-    use crate::{options::OutputMode, stats::ResponseStats};
-    use http::StatusCode;
-    use lychee_lib::{InputSource, ResponseBody, Status, Uri};
-    use std::collections::{HashMap, HashSet};
-    use url::Url;
+    use crate::formatters::stats::{StatsFormatter, get_dummy_stats};
+    use crate::options::OutputMode;
+    use regex::Regex;
 
     use super::*;
 
     #[test]
     fn test_formatter() {
-        // A couple of dummy successes
-        let mut success_map: HashMap<InputSource, HashSet<ResponseBody>> = HashMap::new();
-
-        success_map.insert(
-            InputSource::RemoteUrl(Box::new(Url::parse("https://example.com").unwrap())),
-            HashSet::from_iter(vec![ResponseBody {
-                uri: Uri::from(Url::parse("https://example.com").unwrap()),
-                status: Status::Ok(StatusCode::OK),
-            }]),
-        );
-
-        let err1 = ResponseBody {
-            uri: Uri::try_from("https://github.com/mre/idiomatic-rust-doesnt-exist-man").unwrap(),
-            status: Status::Ok(StatusCode::NOT_FOUND),
-        };
-
-        let err2 = ResponseBody {
-            uri: Uri::try_from("https://github.com/mre/boom").unwrap(),
-            status: Status::Ok(StatusCode::INTERNAL_SERVER_ERROR),
-        };
-
-        let mut error_map: HashMap<InputSource, HashSet<ResponseBody>> = HashMap::new();
-        let source = InputSource::RemoteUrl(Box::new(Url::parse("https://example.com").unwrap()));
-        error_map.insert(source, HashSet::from_iter(vec![err1, err2]));
-
-        let stats = ResponseStats {
-            total: 1,
-            successful: 1,
-            errors: 2,
-            unknown: 0,
-            excludes: 0,
-            timeouts: 0,
-            duration_secs: 0,
-            error_map,
-            suggestion_map: HashMap::default(),
-            redirect_map: HashMap::default(),
-            unsupported: 0,
-            redirects: 0,
-            cached: 0,
-            success_map,
-            excluded_map: HashMap::default(),
-            detailed_stats: false,
-        };
-
         let formatter = Compact::new(OutputMode::Plain);
+        let result = formatter.format(get_dummy_stats()).unwrap();
 
-        let result = formatter.format(stats).unwrap().unwrap();
+        // Remove color codes for better readability of the expected result
+        let without_color_codes = Regex::new(r"\u{1b}\[[0-9;]*m")
+            .unwrap()
+            .replace_all(&result, "")
+            .to_string();
 
-        println!("{result}");
+        assert_eq!(
+            without_color_codes,
+            "Issues found in 1 input. Find details below.
 
-        assert!(result.contains("🔍 1 Total"));
-        assert!(result.contains("✅ 1 OK"));
-        assert!(result.contains("🚫 2 Errors"));
+[https://example.com/]:
+[404] https://github.com/mre/idiomatic-rust-doesnt-exist-man | 404 Not Found: Not Found
 
-        assert!(result.contains("[https://example.com/]:"));
-        assert!(
-            result
-                .contains("https://github.com/mre/idiomatic-rust-doesnt-exist-man | 404 Not Found")
+ℹ Suggestions
+https://original.dev/ --> https://suggestion.dev/
+
+🔍 2 Total (in 0s) ✅ 0 OK 🚫 1 Error 🔀 1 Redirects
+
+📊 Per-host Statistics
+────────────────────────────────────────────────────────────
+example.com   │      5 reqs │   60.0% success │      N/A median │   20.0% cached
+"
         );
-        assert!(result.contains("https://github.com/mre/boom | 500 Internal Server Error"));
     }
 }
