@@ -75,11 +75,13 @@ impl Collector {
     pub fn new(root_dir: Option<PathBuf>, base: Option<Base>) -> LycheeResult<Self> {
         let root_dir = match root_dir {
             Some(root_dir) if base.is_some() => Some(root_dir),
-            Some(root_dir) => Some(
-                root_dir
-                    .canonicalize()
-                    .map_err(|e| ErrorKind::InvalidRootDir(root_dir, e))?,
-            ),
+            Some(root_dir) => {
+                let root_dir_exists = root_dir.read_dir().map(|_| ());
+                let root_dir = root_dir_exists
+                    .and_then(|()| std::path::absolute(&root_dir))
+                    .map_err(|e| ErrorKind::InvalidRootDir(root_dir, e))?;
+                Some(root_dir)
+            }
             None => None,
         };
         Ok(Collector {
